/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <zephyr.h>
#include <device.h>
#include <devicetree.h>
#include <drivers/gpio.h>
#include <drivers/spi.h>

/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS   1000

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led0)

#if DT_NODE_HAS_STATUS(LED0_NODE, okay)
#define LED0	DT_GPIO_LABEL(LED0_NODE, gpios)
#define PIN	DT_GPIO_PIN(LED0_NODE, gpios)
#define FLAGS	DT_GPIO_FLAGS(LED0_NODE, gpios)
#else
/* A build error here means your board isn't set up to blink an LED. */
#error "Unsupported board: led0 devicetree alias is not defined"
#define LED0	""
#define PIN	0
#define FLAGS	0
#endif

// change to
#define SPI_DEVICE_NAME		DT_LABEL(DT_NODELABEL(spi1))
#define SPI_GPIO_NAME		DT_LABEL(DT_NODELABEL(gpio0))


#define CS_GPIO_PIN 4
#define BUSY_GPIO_PIN 7
#define RESET_GPIO_PIN 26
#define DC_GPIO_PIN 25


static const struct device * gpio_dev;
static const struct device * spi_dev;


static struct spi_cs_control spi_cs = {
        .gpio_pin = CS_GPIO_PIN,
        .delay = 0,
        .gpio_dt_flags = GPIO_ACTIVE_LOW,
};

static const struct spi_config spi_cfg = {
	.operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB | SPI_MODE_CPOL | SPI_MODE_CPHA,
        .cs=&spi_cs,
	.frequency = 8000000,
	.slave = 0,
};



static int GPIO_Init(void) {

    int err = 0;

    gpio_dev = device_get_binding(SPI_GPIO_NAME);

    if (!gpio_dev) {
        err = -ENXIO;
    } else {
        gpio_pin_configure(gpio_dev, DC_GPIO_PIN , GPIO_OUTPUT);
        gpio_pin_configure(gpio_dev, RESET_GPIO_PIN, GPIO_OUTPUT);
    }
    return err;
}


static int SPI_Init(void) {
    int err = 0;

    spi_dev = device_get_binding(SPI_DEVICE_NAME);

    if (spi_dev == NULL) {
        err = -ENXIO;
    } else {
        spi_cs.gpio_dev = gpio_dev;
    }
    return err;
}

void SPI_sendData(uint8_t data) {
    int err;

    const struct spi_buf tx_buf = {
          .buf = &data,
          .len = sizeof(data)
    };

    const struct spi_buf_set tx = {
          .buffers = &tx_buf,
          .count = 1
    };

    err = spi_transceive(spi_dev, &spi_cfg, &tx, NULL);
    if (err) {
        printk("SPI error: %d\n", err);
    }
}


void main(void)
{
	const struct device *dev;
	bool led_is_on = true;
	int ret;

#ifdef CONFIG_SERIAL
        // disable uart rx, this drops idle current draw without the need to remove the whole serial logging interface
        NRF_UARTE0->TASKS_STOPRX = 1;
#endif

        // initialize the gpio pins for the display
        if (GPIO_Init() != 0) {
            return;
        }

        // initialize the spi interface for the display
        if (SPI_Init() != 0) {
            return;
        }

        // send a byte of data over spi
        SPI_sendData(1);


	dev = device_get_binding(LED0);
	if (dev == NULL) {
		return;
	}

	ret = gpio_pin_configure(dev, PIN, GPIO_OUTPUT_ACTIVE | FLAGS);
	if (ret < 0) {
		return;
	}

	while (1) {
		gpio_pin_set(dev, PIN, (int)led_is_on);
		led_is_on = !led_is_on;
		k_msleep(SLEEP_TIME_MS);
	}
}
